@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -675,9 +675,10 @@ function template_edit() {
 	}
 
 	if (get_request_var('id') > 0) {
-		$readOnly = db_fetch_cell_prepared('SELECT *
+		$readOnly = db_fetch_cell_prepared('SELECT id
 			FROM data_local
-			WHERE data_template_id = ?',
+			WHERE data_template_id = ?
+			LIMIT 1',
 			array(get_request_var('id')));
 	} else {
 		$readOnly = false;
@@ -1099,7 +1100,7 @@ function template() {
 		)
 	);
 
-	$nav = html_nav_bar('data_templates.php?filter=' . get_request_var('filter'), MAX_DISPLAY_PAGES, get_request_var('page'), $rows, $total_rows, sizeof($display_text) + 1, __('Data Templates'), 'page', 'main');
+	$nav = html_nav_bar('data_templates.php?filter=' . get_request_var('filter'), MAX_DISPLAY_PAGES, get_request_var('page'), $rows, $total_rows, cacti_sizeof($display_text) + 1, __('Data Templates'), 'page', 'main');
 
 	form_start('data_templates.php', 'chk');
 
