@@ -55,13 +55,17 @@ function get_matching_nodes() {
 			FROM graph_tree_items AS gti
 			LEFT JOIN host AS h
 			ON h.id=gti.host_id
+			RIGHT JOIN host AS h2
+			ON h2.site_id=gti.site_id
 			LEFT JOIN graph_templates_graph AS gtg
 			ON gtg.local_graph_id=gti.local_graph_id AND gtg.local_graph_id>0
 			WHERE gtg.title_cache LIKE ?
 			OR h.description LIKE ?
+			OR h2.description LIKE ?
 			OR h.hostname LIKE ?
+			OR h2.hostname LIKE ?
 			OR gti.title LIKE ?",
-			array($filter, $filter, $filter, $filter));
+			array($filter, $filter, $filter, $filter, $filter, $filter));
 	} else {
 		$matching = db_fetch_assoc("SELECT parent, graph_tree_id FROM graph_tree_items");
 	}
@@ -439,7 +443,7 @@ case 'preview':
 	/* create filter for sql */
 	$sql_where  = '';
 	if (!isempty_request_var('rfilter')) {
-		$sql_where .= " gtg.title_cache RLIKE " . db_qstr(preg_quote(get_request_var('rfilter')));
+		$sql_where .= " gtg.title_cache RLIKE " . db_qstr(get_request_var('rfilter'));
 	}
 
 	$sql_where .= ($sql_or != '' && $sql_where != '' ? ' AND ':'') . $sql_or;
@@ -685,7 +689,7 @@ case 'list':
 	/* create filter for sql */
 	$sql_where  = '';
 	if (!isempty_request_var('rfilter')) {
-		$sql_where .= " gtg.title_cache RLIKE " . db_qstr(preg_quote(get_request_var('rfilter')));
+		$sql_where .= " gtg.title_cache RLIKE " . db_qstr(get_request_var('rfilter'));
 	}
 
 	if (!isempty_request_var('host_id') && get_request_var('host_id') > 0) {
