@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -114,6 +114,7 @@ switch (get_request_var('action')) {
 		get_filter_request_var('host_id');
 
 		host_reload_query();
+		raise_message('query_reloaded', __('Data Query Re-indexed.'), MESSAGE_LEVEL_INFO);
 
 		header('Location: host.php?header=false&action=edit&id=' . get_request_var('host_id'));
 		break;
@@ -121,6 +122,7 @@ switch (get_request_var('action')) {
 		get_filter_request_var('host_id');
 
 		host_reload_query();
+		raise_message('query_reloaded', __('Device Data Query Re-indexed.  Verbose output displayed.'), MESSAGE_LEVEL_INFO);
 
 		header('Location: host.php?header=' . (isset_request_var('header') && get_nfilter_request_var('header') == 'true' ? 'true':'false') . '&action=edit&id=' . get_request_var('host_id') . '&display_dq_details=true');
 		break;
@@ -139,11 +141,26 @@ switch (get_request_var('action')) {
 		break;
 	case 'enable_debug':
 		enable_device_debug(get_filter_request_var('host_id'));
+		raise_message('enable_debug', __('Device Debugging Enabled for Device.'), MESSAGE_LEVEL_INFO);
+
 		header('Location: host.php?header=false&action=edit&id=' . get_request_var('host_id'));
 
 		break;
 	case 'disable_debug':
 		disable_device_debug(get_filter_request_var('host_id'));
+		raise_message('disable_debug', __('Device Debugging Disabled for Device.'), MESSAGE_LEVEL_INFO);
+
+		header('Location: host.php?header=false&action=edit&id=' . get_request_var('host_id'));
+
+		break;
+	case 'repopulate':
+		if (get_filter_request_var('host_id') > 0) {
+			push_out_host(get_request_var('host_id'));
+			raise_message('repopulate_message', __('Poller Cache for Device Refreshed.'), MESSAGE_LEVEL_INFO);
+		} else {
+			raise_message('repopulate_error', __('ERROR: Invalid Device ID.'), MESSAGE_LEVEL_ERROR);
+		}
+
 		header('Location: host.php?header=false&action=edit&id=' . get_request_var('host_id'));
 
 		break;
@@ -250,7 +267,7 @@ function form_save() {
 				get_nfilter_request_var('snmp_engine_id'), get_nfilter_request_var('max_oids'),
 				get_nfilter_request_var('device_threads'), get_nfilter_request_var('poller_id'),
 				get_nfilter_request_var('site_id'), get_nfilter_request_var('external_id'),
-				get_nfilter_request_var('location'));
+				get_nfilter_request_var('location'), get_nfilter_request_var('bulk_walk_size'));
 
 			if ($host_id !== false) {
 				api_plugin_hook_function('host_save', array('host_id' => $host_id));
@@ -616,6 +633,7 @@ function host_edit() {
 
 	$header_label = __('Device [new]');
 	$debug_link   = '';
+	$repop_link   = '';
 	if (!isempty_request_var('id')) {
 		$_SESSION['cur_device_id'] = get_request_var('id');
 
@@ -631,6 +649,8 @@ function host_edit() {
 			} else {
 				$debug_link = "<span class='linkMarker'>*</span><a class='hyperLink' href='" . html_escape('host.php?action=enable_debug&host_id=' . $host['id']) . "'>" . __('Enable Device Debug') . "</a><br>";
 			}
+
+			$repop_link = "<span class='linkMarker'>*</span><a class='hyperLink' href='" . html_escape('host.php?action=repopulate&host_id=' . $host['id']) . "'>" . __('Repopulate Poller Cache') . "</a><br>";
 		}
 	} else {
 		$_SESSION['cur_device_id'] = 0;
@@ -648,6 +668,7 @@ function host_edit() {
 					<span class='linkMarker'>*</span><a class='hyperLink' href='<?php print html_escape('graphs_new.php?reset=true&host_id=' . $host['id']);?>'><?php print __('Create Graphs for this Device');?></a><br>
 					<span class='linkMarker'>*</span><a class='hyperLink' href='<?php print html_escape('host.php?action=reindex&host_id=' . $host['id']);?>'><?php print __('Re-Index Device');?></a><br>
 					<?php print $debug_link;?>
+					<?php print $repop_link;?>
 					<span class='linkMarker'>*</span><a class='hyperLink' href='<?php print html_escape('data_sources.php?reset=true&host_id=' . $host['id'] . '&ds_rows=30&filter=&template_id=-1&method_id=-1&page=1');?>'><?php print __('Data Source List');?></a><br>
 					<span class='linkMarker'>*</span><a class='hyperLink' href='<?php print html_escape('graphs.php?reset=true&host_id=' . $host['id'] . '&graph_rows=30&filter=&template_id=-1&page=1');?>'><?php print __('Graph List');?></a>
 					<?php api_plugin_hook('device_edit_top_links'); ?>
@@ -682,32 +703,6 @@ function host_edit() {
 
 	device_javascript();
 
-	if ((isset_request_var('display_dq_details')) && (isset($_SESSION['debug_log']['data_query']))) {
-		$dbg_copy_uid = generate_hash();
-		?>
-		<div id='dqdebug' class='cactiTable'>
-			<div id='clipboardHeader<?php print $dbg_copy_uid;?>'>
-				<div class='cactiTableTitle'>
-					<span style='padding:3px;'><?php print __('Data Query Debug Information');?></span>
-				</div>
-				<div class='cactiTableButton'>
-					<span>
-						<a class='linkCopyDark cactiTableCopy' id='copyToClipboard<?php print $dbg_copy_uid;?>'><?php print __('Copy');?></a>
-						<a id='dbghide' class='fa fa-times' href='#'><?php print __('Hide');?></a>
-					</span>
-				</div>
-				<table class='cactiTable' id='clipboardData<?php print $dbg_copy_uid;?>'>
-					<tr class='tableRow'>
-						<td class='debug'>
-							<span><?php print debug_log_return('data_query');?></span>
-						</td>
-					</tr>
-				</table>
-			</div>
-		</div>
-		<?php
-	}
-
 	if (!empty($host['id'])) {
 		html_start_box(__('Associated Graph Templates'), '100%', '', '3', 'center', '');
 
@@ -807,6 +802,32 @@ function host_edit() {
 		<?php
 		html_end_box();
 
+		if ((isset_request_var('display_dq_details')) && (isset($_SESSION['debug_log']['data_query']))) {
+			$dbg_copy_uid = generate_hash();
+			?>
+			<div id='dqdebug' class='cactiTable'>
+				<div id='clipboardHeader<?php print $dbg_copy_uid;?>'>
+					<div class='cactiTableTitle'>
+						<span style='padding:3px;'><?php print __('Data Query Debug Information');?></span>
+					</div>
+					<div class='cactiTableButton'>
+						<span>
+							<a class='linkCopyDark cactiTableCopy' id='copyToClipboard<?php print $dbg_copy_uid;?>'><?php print __('Copy');?></a>
+							<a id='dbghide' class='fa fa-times' href='#'><?php print __('Hide');?></a>
+						</span>
+					</div>
+					<table class='cactiTable' id='clipboardData<?php print $dbg_copy_uid;?>'>
+						<tr class='tableRow'>
+							<td class='debug'>
+								<span><?php print debug_log_return('data_query');?></span>
+							</td>
+						</tr>
+					</table>
+				</div>
+			</div>
+			<?php
+		}
+
 		html_start_box(__('Associated Data Queries'), '100%', '', '3', 'center', '');
 
 		html_header(
@@ -1142,6 +1163,15 @@ function device_javascript() {
 		setPing();
 	}
 
+	function hostPageLoad(strURL) {
+		var scrollTop = $(window).scrollTop();
+		$.get(strURL, function(data) {
+			$('#main').html(data);
+			applySkin();
+			$(window).scrollTop(scrollTop);
+		});
+	}
+
 	$(function() {
 		// Need to set this for global snmpv3 functions to remain sane between edits
 		snmp_security_initialized = false;
@@ -1175,23 +1205,22 @@ function device_javascript() {
 		$('[id^="reload"]').click(function(data) {
 			$(this).addClass('fa-spin');
 			strURL = 'host.php?action=query_reload&id='+$(this).attr('data-id')+'&host_id='+$('#id').val()+'&nostate=true';
-			loadPageNoHeader(strURL, true);
+			hostPageLoad(strURL);
 		});
 
 		$('[id^="verbose"]').click(function(data) {
-			$(this).addClass('fa-spin');
-			strURL = 'host.php?action=query_verbose&id='+$(this).attr('data-id')+'&host_id='+$('#id').val()+'&nostate=true';
+			var strURL = 'host.php?action=query_verbose&id='+$(this).attr('data-id')+'&host_id='+$('#id').val()+'&nostate=true';
 			loadPageNoHeader(strURL, true);
 		});
 
 		$('[id^="remove"]').click(function(data) {
-			strURL = 'host.php?action=query_remove&id='+$(this).attr('data-id')+'&host_id='+$('#id').val()+'&nostate=true';
-			loadPageNoHeader(strURL, true);
+			var strURL = 'host.php?action=query_remove&id='+$(this).attr('data-id')+'&host_id='+$('#id').val()+'&nostate=true';
+			hostPageLoad(strURL);
 		});
 
 		$('[id^="gtremove"]').click(function(data) {
 			strURL = 'host.php?action=gt_remove&id='+$(this).attr('data-id')+'&host_id='+$('#id').val()+'&nostate=true';
-			loadPageNoHeader(strURL, true);
+			hostPageLoad(strURL);
 		});
 
 		$('#add_dq').click(function() {
@@ -1217,6 +1246,11 @@ function device_javascript() {
 			$('#dqdebug').empty().fadeOut('fast');
 		});
 
+		if ($('#dbghide').length) {
+			var dbgloc = parseInt($('#dbghide').offset().top - $('.breadCrumbBar').outerHeight() - $('.cactiPageHead').outerHeight());
+			$('.cactiConsoleContentArea').scrollTop(dbgloc);
+		}
+
 		$('[id$="spacer"]').click(function() {
 			changeHostForm();
 		});
@@ -1236,7 +1270,6 @@ function device_javascript() {
 			.done(function(data) {
 				$('#ping_results').html(data);
 				hostInfoHeight = $('.hostInfoHeader').height();
-				$('#navigation_right, #main').scrollTop(0);
 			})
 			.fail(function(data) {
 				getPresentHTTPError(data);
@@ -1279,6 +1312,7 @@ function host_validate_vars() {
 			),
 		'location' => array(
 			'filter' => FILTER_CALLBACK,
+			'pageset' => true,
 			'default' => '-1',
 			'options' => array('options' => 'sanitize_search_string')
 			),
@@ -1321,6 +1355,8 @@ function host_validate_vars() {
 }
 
 function get_device_records(&$total_rows, $rows) {
+	$sql_where = '';
+
 	/* form the 'where' clause for our main sql query */
 	if (get_request_var('filter') != '') {
 		$sql_where = 'WHERE (deleted = "" AND (
@@ -1421,58 +1457,6 @@ function host() {
 		$rows = get_request_var('rows');
 	}
 
-	?>
-	<script type='text/javascript'>
-
-	function applyFilter() {
-		strURL  = 'host.php';
-		strURL += '?host_status=' + $('#host_status').val();
-		strURL += '&host_template_id=' + $('#host_template_id').val();
-		strURL += '&site_id=' + $('#site_id').val();
-		strURL += '&poller_id=' + $('#poller_id').val();
-		strURL += '&location=' + $('#location').val();
-		strURL += '&rows=' + $('#rows').val();
-		strURL += '&filter=' + $('#filter').val();
-		strURL += '&header=false';
-		loadPageNoHeader(strURL);
-	}
-
-	function clearFilter() {
-		strURL = 'host.php?clear=1&header=false';
-		loadPageNoHeader(strURL);
-	}
-
-	function exportRecords() {
-		strURL = 'host.php?action=export';
-		document.location = strURL;
-	}
-
-	$(function() {
-		$('#rows, #site_id, #poller_id, #location, #host_template_id, #host_status').change(function() {
-			applyFilter();
-		});
-
-		$('#refresh').click(function() {
-			applyFilter();
-		});
-
-		$('#clear').click(function() {
-			clearFilter();
-		});
-
-		$('#export').click(function() {
-			exportRecords();
-		});
-
-		$('#form_devices').submit(function(event) {
-			event.preventDefault();
-			applyFilter();
-		});
-	});
-
-	</script>
-	<?php
-
 	if (get_request_var('host_template_id') > 0) {
 		$url = 'host.php?action=edit&host_template_id=' . get_request_var('host_template_id');
 	} else {
@@ -1487,6 +1471,7 @@ function host() {
 		<form id='form_devices' action='host.php'>
 			<table class='filterTable'>
 				<tr>
+					<?php api_plugin_hook('device_filter_start'); ?>
 					<td>
 						<?php print __('Site');?>
 					</td>
@@ -1499,7 +1484,7 @@ function host() {
 
 							if (cacti_sizeof($sites)) {
 								foreach ($sites as $site) {
-									print "<option value='" . $site['id'] . "'"; if (get_request_var('site_id') == $site['id']) { print ' selected'; } print '>' . html_escape($site['name']) . "</option>";
+									print "<option value='" . $site['id'] . "'"; if (get_request_var('site_id') == $site['id']) { print ' selected'; } print '>' . html_escape($site['name']) . '</option>';
 								}
 							}
 							?>
@@ -1516,7 +1501,7 @@ function host() {
 
 							if (cacti_sizeof($pollers)) {
 								foreach ($pollers as $poller) {
-									print "<option value='" . $poller['id'] . "'"; if (get_request_var('poller_id') == $poller['id']) { print ' selected'; } print '>' . html_escape($poller['name']) . "</option>";
+									print "<option value='" . $poller['id'] . "'"; if (get_request_var('poller_id') == $poller['id']) { print ' selected'; } print '>' . html_escape($poller['name']) . '</option>';
 								}
 							}
 							?>
@@ -1534,7 +1519,7 @@ function host() {
 
 							if (cacti_sizeof($host_templates)) {
 								foreach ($host_templates as $host_template) {
-									print "<option value='" . $host_template['id'] . "'"; if (get_request_var('host_template_id') == $host_template['id']) { print ' selected'; } print '>' . html_escape($host_template['name']) . "</option>";
+									print "<option value='" . $host_template['id'] . "'"; if (get_request_var('host_template_id') == $host_template['id']) { print ' selected'; } print '>' . html_escape($host_template['name']) . '</option>';
 								}
 							}
 							?>
@@ -1553,14 +1538,14 @@ function host() {
 							} else {
 								$sql_where = '';
 							}
-							$locations = db_fetch_assoc("SELECT DISTINCT IF(location='', '" . __('Undefined') . "', location) AS location
+							$locations = db_fetch_assoc("SELECT DISTINCT IF(IFNULL(host.location,'') = '', '" . __('Undefined') . "', location) AS location
 								FROM host
 								$sql_where
 								ORDER BY location");
 
 							if (cacti_sizeof($locations)) {
 								foreach ($locations as $l) {
-									print "<option value='" . $l['location'] . "'"; if (get_request_var('location') == $l['location']) { print ' selected'; } print '>' . html_escape($l['location']) . "</option>";
+									print "<option value='" . $l['location'] . "'"; if (get_request_var('location') == $l['location']) { print ' selected'; } print '>' . html_escape($l['location']) . '</option>';
 								}
 							}
 							?>
@@ -1568,7 +1553,7 @@ function host() {
 					</td>
 					<td>
 						<span>
-							<input type='button' class='ui-button ui-corner-all ui-widget' id='refresh' value='<?php print __('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
+							<input type='submit' class='ui-button ui-corner-all ui-widget' id='go' value='<?php print __('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
 							<input type='button' class='ui-button ui-corner-all ui-widget' id='clear' value='<?php print __('Clear');?>' title='<?php print __esc('Clear Filters');?>'>
 							<input type='button' class='ui-button ui-corner-all ui-widget' id='export' value='<?php print __('Export');?>' title='<?php print __esc('Export Devices');?>'>
 						</span>
@@ -1607,15 +1592,62 @@ function host() {
 							<?php
 							if (cacti_sizeof($item_rows)) {
 								foreach ($item_rows as $key => $value) {
-									print "<option value='" . $key . "'"; if (get_request_var('rows') == $key) { print ' selected'; } print '>' . html_escape($value) . "</option>";
+									print "<option value='" . $key . "'"; if (get_request_var('rows') == $key) { print ' selected'; } print '>' . html_escape($value) . '</option>';
 								}
 							}
 							?>
 						</select>
 					</td>
+					<?php api_plugin_hook('device_filter_end'); ?>
 				</tr>
 			</table>
 		</form>
+		<script type='text/javascript'>
+
+		function applyFilter() {
+			strURL  = 'host.php';
+			strURL += '?host_status=' + $('#host_status').val();
+			strURL += '&host_template_id=' + $('#host_template_id').val();
+			strURL += '&site_id=' + $('#site_id').val();
+			strURL += '&poller_id=' + $('#poller_id').val();
+			strURL += '&location=' + $('#location').val();
+			strURL += '&rows=' + $('#rows').val();
+			strURL += '&filter=' + escape($('#filter').val());
+			strURL += '&header=false';
+			loadPageNoHeader(strURL);
+		}
+
+		function clearFilter() {
+			strURL = 'host.php?clear=1&header=false';
+			loadPageNoHeader(strURL);
+		}
+
+		function exportRecords() {
+			strURL = 'host.php?action=export';
+			document.location = strURL;
+			Pace.stop();
+		}
+
+		$(function() {
+			$('#rows, #site_id, #poller_id, #location, #host_template_id, #host_status').change(function() {
+				applyFilter();
+			});
+
+			$('#clear').click(function() {
+				clearFilter();
+			});
+
+			$('#export').click(function() {
+				exportRecords();
+			});
+
+			$('#form_devices').submit(function(event) {
+				event.preventDefault();
+				applyFilter();
+			});
+		});
+
+		</script>
 		</td>
 	</tr>
 	<?php
@@ -1697,6 +1729,9 @@ function host() {
 		)
 	);
 
+	$display_text_size = sizeof($display_text);
+	$display_text = api_plugin_hook_function('device_display_text', $display_text);
+
 	$hosts = get_device_records($total_rows, $rows);
 
 	$nav = html_nav_bar('host.php?filter=' . get_request_var('filter'), MAX_DISPLAY_PAGES, get_request_var('page'), $rows, $total_rows, cacti_sizeof($display_text) + 1, __('Devices'), 'page', 'main');
@@ -1709,7 +1744,9 @@ function host() {
 
 	html_header_sort_checkbox($display_text, get_request_var('sort_column'), get_request_var('sort_direction'), false);
 
-	if (cacti_sizeof($hosts)) {
+	if (sizeof($display_text) != $display_text_size && cacti_sizeof($hosts)) {//display_text changed
+		api_plugin_hook_function('device_table_replace', $hosts);
+	} else if (cacti_sizeof($hosts)) {
 		foreach ($hosts as $host) {
 			if ($host['disabled'] == '' &&
 				($host['status'] == HOST_RECOVERING || $host['status'] == HOST_UP) &&
