@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -317,6 +317,14 @@ function boost_graph_cache_check($local_graph_id, $rra_id, $rrdtool_pipe, &$grap
 		return false;
 	}
 
+	/* This is a realtime graph */
+	if (isset($graph_data_array['export_realtime'])) {
+		/* restore original error handler */
+		restore_error_handler();
+
+		return false;
+	}
+
 	/* if we are just printing the rrd command return */
 	if (isset($graph_data_array['print_source'])) {
 		/* restore original error handler */
@@ -584,21 +592,22 @@ function boost_timer_get_overhead() {
 	return (microtime(true) - $start);
 }
 
-/* boost_get_arch_table_name - returns current archive boost table or false if no arch table is present currently */
-function boost_get_arch_table_name() {
-	$tables = db_fetch_assoc("SELECT table_name AS name
-		FROM information_schema.tables
-		WHERE table_schema = SCHEMA()
-		AND table_name LIKE 'poller_output_boost_arch_%'");
+/* boost_get_arch_table_names - returns current archive boost tables or false if no arch table is present currently */
+function boost_get_arch_table_names() {
+	$tableNames = array_rekey(
+		db_fetch_assoc("SELECT TABLE_NAME AS name
+			FROM information_schema.TABLES
+			WHERE TABLE_SCHEMA = SCHEMA()
+			AND TABLE_NAME LIKE 'poller_output_boost_arch_%'
+			AND TABLE_ROWS > 0"),
+		'name', 'name'
+	);
 
-	foreach($tables as $table) {
-		$rows = db_fetch_cell('SELECT COUNT(local_data_id) FROM ' . $table['name']);
-		if (is_numeric($rows) && intval($rows) > 0) {
-			return $table['name'];
-		}
+	if (!cacti_sizeof($tableNames)) {
+		return false;
+	} else {
+		return $tableNames;
 	}
-
-	return false;
 }
 
 /* boost_process_poller_output - grabs data from the 'poller_output' table and feeds the *completed*
@@ -709,7 +718,6 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 		/* initialize some variables */
 		$rrd_tmpl     = '';
 		$rrd_path     = '';
-		$initial_time = 0;
 		$outlen       = 0;
 
 		boost_timer('results_cycle', BOOST_TIMER_START);
@@ -730,7 +738,7 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 				/* update the rrd for the previous local_data_id */
 				if ($vals_in_buffer) {
 					boost_timer('rrdupdate', BOOST_TIMER_START);
-					$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $initial_time, $outbuf, $rrdtool_pipe);
+					$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $outbuf, $rrdtool_pipe);
 					boost_timer('rrdupdate', BOOST_TIMER_END);
 
 					$vals_in_buffer = 0;
@@ -754,7 +762,6 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 
 				$local_data_id  = $item['local_data_id'];
 				$time           = $item['timestamp'];
-				$initial_time   = $time;
 
 				if ($time < $last_update && cacti_version_compare(get_rrdtool_version(), '1.5', '<')) {
 					$output = '';
@@ -778,7 +785,7 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 			if ($time != $item['timestamp']) {
 				if ($outlen > $upd_string_len) {
 					boost_timer('rrdupdate', BOOST_TIMER_START);
-					$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $initial_time, $outbuf, $rrdtool_pipe);
+					$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $outbuf, $rrdtool_pipe);
 					boost_timer('rrdupdate', BOOST_TIMER_END);
 
 					$outbuf         = '';
@@ -878,7 +885,7 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 		/* process the last rrdupdate if applicable */
 		if ($vals_in_buffer) {
 			boost_timer('rrdupdate', BOOST_TIMER_START);
-			$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $initial_time, $outbuf, $rrdtool_pipe);
+			$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $outbuf, $rrdtool_pipe);
 			boost_timer('rrdupdate', BOOST_TIMER_END);
 
 			/* check return status for delete operation */
@@ -894,7 +901,7 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 
 		if (cacti_count($arch_tables)) {
 			foreach($arch_tables as $table) {
-				db_execute_prepared('DELETE FROM ' . $table['name'] . '
+				db_execute_prepared('DELETE IGNORE FROM ' . $table['name'] . '
 					WHERE local_data_id = ?',
 					array($local_data_id));
 			}
@@ -1024,7 +1031,7 @@ function boost_get_rrd_filename_and_template($local_data_id) {
 	return array('rrd_path' => $rrd_path, 'rrd_template' => trim($rrd_template));
 }
 
-function boost_rrdtool_function_create($local_data_id, $initial_time, $show_source, &$rrdtool_pipe) {
+function boost_rrdtool_function_create($local_data_id, $show_source, &$rrdtool_pipe) {
 	global $config;
 
 	/**
@@ -1078,11 +1085,8 @@ function boost_rrdtool_function_create($local_data_id, $initial_time, $show_sour
 		return false;
 	}
 
-	/* back off the initial time to allow updates */
-	$initial_time -= 300;
-
 	/* create the "--step" line */
-	$create_ds = RRD_NL . '--start ' . $initial_time . ' --step '. $rras[0]['rrd_step'] . ' ' . RRD_NL;
+	$create_ds = RRD_NL . '--start 0 --step '. $rras[0]['rrd_step'] . ' ' . RRD_NL;
 
 	/* query the data sources to be used in this .rrd file */
 	$data_sources = db_fetch_assoc_prepared('SELECT
@@ -1183,7 +1187,7 @@ function boost_rrdtool_function_create($local_data_id, $initial_time, $show_sour
    @arg $rrd_path      - the path to the RRD file
    @arg $rrd_update_template  - the order in which values need to be added
    @arg $rrd_update_values    - values to include in the database */
-function boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_update_template, $initial_time, &$rrd_update_values, &$rrdtool_pipe) {
+function boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_update_template, &$rrd_update_values, &$rrdtool_pipe) {
 
 	/* lets count the number of rrd files processed */
 	$rrds_processed = 0;
@@ -1211,7 +1215,7 @@ function boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_update_te
 
 		// Check for a Data Source that has been removed
 		if ($ds_exists) {
-			$valid_entry = boost_rrdtool_function_create($local_data_id, $initial_time, false, $rrdtool_pipe);
+			$valid_entry = boost_rrdtool_function_create($local_data_id, false, $rrdtool_pipe);
 		} else {
 			return 'OK';
 		}
@@ -1281,21 +1285,23 @@ function boost_update_snmp_statistics () {
 	$engine = '';
 	$max_data_length = 0;
 
-	foreach($boost_table_status as $table) {
-		if ($table['TABLE_NAME'] == 'poller_output_boost') {
-			$pending_records += $table['TABLE_ROWS'];
-		} else {
-			$arch_records += $table['TABLE_ROWS'];
+	if (cacti_sizeof($boost_table_status)) {
+		foreach($boost_table_status as $table) {
+			if ($table['TABLE_NAME'] == 'poller_output_boost') {
+				$pending_records += $table['TABLE_ROWS'];
+			} else {
+				$arch_records += $table['TABLE_ROWS'];
+			}
+			$data_length += $table['DATA_LENGTH'];
+			$data_length -= $table['DATA_FREE'];
+			$engine = $table['ENGINE'];
+			$max_data_length = $table['MAX_DATA_LENGTH'];
 		}
-		$data_length += $table['DATA_LENGTH'];
-		$data_length -= $table['DATA_FREE'];
-		$engine = $table['ENGINE'];
-		$max_data_length = $table['MAX_DATA_LENGTH'];
 	}
+
 	$total_records = $pending_records + $arch_records;
 	$avg_row_length = ($total_records ? intval($data_length / $total_records) : 0);
 
-
 	if (strcmp($engine, 'MEMORY') == 0) {
 		$max_length = db_fetch_cell('SELECT MAX(LENGTH(output)) FROM poller_output_boost');
 		$max_table_allowed = $max_data_length;
