@@ -1120,9 +1120,13 @@ function draw_graph_items_list($item_list, $filename, $url_data, $disable_contro
 
 	print "<tr class='tableHeader'>";
 		DrawMatrixHeaderItem(__('Graph Item'),'',1);
+		DrawMatrixHeaderItem(__('#'), '', 1);
 		DrawMatrixHeaderItem(__('Data Source'),'',1);
 		DrawMatrixHeaderItem(__('Graph Item Type'),'',1);
 		DrawMatrixHeaderItem(__('CF Type'),'',1);
+		DrawMatrixHeaderItem(__('GPrint'),'',1);
+		DrawMatrixHeaderItem(__('CDEF'),'',1);
+		DrawMatrixHeaderItem(__('VDEF'),'',1);
 		DrawMatrixHeaderItem(__('Alpha %'),'',1);
 		DrawMatrixHeaderItem(__('Item Color'),'',4);
 	print '</tr>';
@@ -1163,6 +1167,7 @@ function draw_graph_items_list($item_list, $filename, $url_data, $disable_contro
 			print __('Item # %d', ($i+1));
 			if ($disable_controls == false) { print '</a>'; }
 			print '</td>';
+			print '<td>' . $item['sequence'] . '</td>';
 
 			if (empty($item['data_source_name'])) {
 				$item['data_source_name'] = __('No Source');
@@ -1206,6 +1211,16 @@ function draw_graph_items_list($item_list, $filename, $url_data, $disable_contro
 				print '<td>' . __('N/A') . '</td>';
 			}
 
+			print "<td style='$this_row_style'>";
+			print $item['gprint_name'];
+			print "</td>";
+			print "<td style='$this_row_style'>";
+			print $item['cdef_name'];
+			print "</td>";
+			print "<td style='$this_row_style'>";
+			print $item['vdef_name'];
+			print "</td>";
+
 			/* alpha type */
 			if (preg_match('/(AREA|STACK|TICK|LINE[123])/', $_graph_type_name)) {
 				print "<td style='$this_row_style'>" . round((hexdec($item['alpha'])/255)*100) . '%</td>';
@@ -2415,7 +2430,7 @@ function html_common_header($title, $selectedTheme = '') {
 	<meta name='description' content='Monitoring tool of the Internet'>
 	<meta name='mobile-web-app-capable' content='yes'>
 	<meta name="theme-color" content="#161616"/>
-	<meta http-equiv="Content-Security-Policy" content="default-src *; img-src 'self' <?php print $alternates;?> data: blob:; style-src 'self' 'unsafe-inline' <?php print $alternates;?>; script-src 'self' <?php print html_escape($script_policy);?> 'unsafe-inline' <?php print $alternates;?>; worker-src 'self'">
+	<meta http-equiv="Content-Security-Policy" content="default-src *; img-src 'self' <?php print $alternates;?> data: blob:; style-src 'self' 'unsafe-inline' <?php print $alternates;?>; script-src 'self' <?php print html_escape($script_policy);?> 'unsafe-inline' <?php print $alternates;?>; worker-src 'self' <?php print $alternates;?>;">
 	<meta name='robots' content='noindex,nofollow'>
 	<title><?php print $title; ?></title>
 	<meta http-equiv='Content-Type' content='text/html;charset=utf-8'>
