@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -131,14 +131,10 @@ function import_xml_data(&$xml_data, $import_as_new, $profile_id, $remove_orphan
 			: 'hash'
 		);
 		$db_table = $db_info['table'];
-		$hash_cache_sql_union_selects[] = "
-			SELECT
-				'$hash_type' AS type,
-				`$db_id_field` AS id,
-				`$db_hash_field` AS hash
-			FROM
-				`$db_table`
-		";
+
+		$hash_cache_sql_union_selects[] = "SELECT '$hash_type' AS type, $db_id_field AS id, $db_hash_field AS hash
+			FROM `$db_table`
+			WHERE $db_hash_field != ''";
 	}
 
 	$hash_cache_sql = implode(' UNION ALL ', $hash_cache_sql_union_selects);
@@ -257,8 +253,6 @@ function import_package($xmlfile, $profile_id = 1, $remove_orphans = false, $pre
 
 	$preview_only = $preview;
 
-	@ini_set('zlib.output_compression', '0');
-
 	/* set new timeout and memory settings */
 	if ($limitex) {
 		ini_set('max_execution_time', '50');
@@ -1664,6 +1658,12 @@ function xml_to_data_input_method($hash, &$xml_array, &$hash_cache) {
 	$save['hash'] = $hash;
 
 	foreach ($fields_data_input_edit as $field_name => $field_array) {
+		// Work around for templates exported during
+		// period where there was a bug in 1.2.13-1.2.15
+		if ($field_name == 'fname') {
+			$field_name = 'name';
+		}
+
 		/* make sure this field exists in the xml array first */
 		if (isset($xml_array[$field_name])) {
 			if ($field_name == 'input_string' && import_is_base64_encoded($xml_array[$field_name])) {
@@ -1733,6 +1733,12 @@ function xml_to_data_input_method($hash, &$xml_array, &$hash_cache) {
 			$save['data_input_id'] = $data_input_id;
 
 			foreach ($fields_data_input_field_edit as $field_name => $field_array) {
+				// Work around for templates exported during
+				// period where there was a bug in 1.2.13-1.2.15
+				if ($field_name == 'fname') {
+					$field_name = 'name';
+				}
+
 				/* make sure this field exists in the xml array first */
 				if (isset($item_array[$field_name])) {
 					$save[$field_name] = xml_character_decode($item_array[$field_name]);
