@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -249,11 +249,11 @@ class spikekill {
 
 		if ($this->method == 'float' || $this->method == 'fill') {
 			if (!is_numeric($this->out_start)) {
-				$this->out_start == strtotime($this->out_start);
+				$this->out_start = strtotime($this->out_start);
 			}
 
 			if (!is_numeric($this->out_end)) {
-				$this->out_end == strtotime($this->out_end);
+				$this->out_end = strtotime($this->out_end);
 			}
 
 			if ($this->out_start === false || $this->out_end === false) {
