@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -35,6 +35,10 @@ function update_replication_crc($poller_id, $variable) {
 }
 
 function repopulate_poller_cache() {
+	global $config;
+
+	include_once($config['library_path'] . '/api_data_source.php');
+
 	$poller_data    = db_fetch_assoc('SELECT ' . SQL_NO_CACHE . ' dl.*, h.poller_id
 		FROM data_local AS dl
 		INNER JOIN host AS h
@@ -100,6 +104,10 @@ function repopulate_poller_cache() {
 }
 
 function update_poller_cache_from_query($host_id, $data_query_id, $local_data_ids) {
+	global $config;
+
+	include_once($config['library_path'] . '/api_data_source.php');
+
 	$poller_data = db_fetch_assoc_prepared('SELECT ' . SQL_NO_CACHE . ' *
 		FROM data_local
 		WHERE host_id = ?
@@ -209,14 +217,11 @@ function update_poller_cache($data_source, $commit = false) {
 		}
 
 		if ($data_input['active'] == 'on') {
-			if (($data_input['type_id'] == DATA_INPUT_TYPE_SCRIPT) || ($data_input['type_id'] == DATA_INPUT_TYPE_PHP_SCRIPT_SERVER)) { /* script */
-				/* fall back to non-script server actions if the user is running a version of php older than 4.3 */
-				if (($data_input['type_id'] == DATA_INPUT_TYPE_PHP_SCRIPT_SERVER) && (function_exists('proc_open'))) {
+			if (($data_input['type_id'] == DATA_INPUT_TYPE_SCRIPT) ||
+				($data_input['type_id'] == DATA_INPUT_TYPE_PHP_SCRIPT_SERVER)) {
+				if ($data_input['type_id'] == DATA_INPUT_TYPE_PHP_SCRIPT_SERVER) {
 					$action = POLLER_ACTION_SCRIPT_PHP;
 					$script_path = get_full_script_path($data_source['id']);
-				} elseif (($data_input['type_id'] == DATA_INPUT_TYPE_PHP_SCRIPT_SERVER) && (!function_exists('proc_open'))) {
-					$action = POLLER_ACTION_SCRIPT;
-					$script_path = read_config_option('path_php_binary') . ' -q ' . get_full_script_path($data_source['id']);
 				} else {
 					$action = POLLER_ACTION_SCRIPT;
 					$script_path = get_full_script_path($data_source['id']);
@@ -241,7 +246,7 @@ function update_poller_cache($data_source, $commit = false) {
 				}
 
 				$poller_items[] = api_poller_cache_item_add($data_source['host_id'], array(), $data_source['id'], $data_input['rrd_step'], $action, $data_source_item_name, 1, $script_path);
-			} elseif ($data_input['type_id'] == DATA_INPUT_TYPE_SNMP) { /* snmp */
+			} elseif ($data_input['type_id'] == DATA_INPUT_TYPE_SNMP) {
 				/* get the host override fields */
 				$data_template_id = db_fetch_cell_prepared('SELECT ' . SQL_NO_CACHE . ' data_template_id
 					FROM data_template_data
@@ -290,7 +295,7 @@ function update_poller_cache($data_source, $commit = false) {
 					array($data_source['id']));
 
 				$poller_items[] = api_poller_cache_item_add($data_source['host_id'], $host_fields, $data_source['id'], $data_input['rrd_step'], 0, get_data_source_item_name($data_template_rrd_id), 1, (isset($host_fields['snmp_oid']) ? $host_fields['snmp_oid'] : ''));
-			} elseif ($data_input['type_id'] == DATA_INPUT_TYPE_SNMP_QUERY) { /* snmp query */
+			} elseif ($data_input['type_id'] == DATA_INPUT_TYPE_SNMP_QUERY) {
 				$snmp_queries = get_data_query_array($data_source['snmp_query_id']);
 
 				/* get the host override fields */
@@ -350,7 +355,8 @@ function update_poller_cache($data_source, $commit = false) {
 						}
 					}
 				}
-			} elseif (($data_input['type_id'] == DATA_INPUT_TYPE_SCRIPT_QUERY) || ($data_input['type_id'] == DATA_INPUT_TYPE_QUERY_SCRIPT_SERVER)) { /* script query */
+			} elseif (($data_input['type_id'] == DATA_INPUT_TYPE_SCRIPT_QUERY) ||
+				($data_input['type_id'] == DATA_INPUT_TYPE_QUERY_SCRIPT_SERVER)) {
 				$script_queries = get_data_query_array($data_source['snmp_query_id']);
 
 				/* get the host override fields */
@@ -400,8 +406,7 @@ function update_poller_cache($data_source, $commit = false) {
 						if (isset($script_queries['fields'][$output['snmp_field_name']]['query_name'])) {
 							$identifier = $script_queries['fields'][$output['snmp_field_name']]['query_name'];
 
-							/* fall back to non-script server actions if the user is running a version of php older than 4.3 */
-							if (($data_input['type_id'] == DATA_INPUT_TYPE_QUERY_SCRIPT_SERVER) && (function_exists('proc_open'))) {
+							if ($data_input['type_id'] == DATA_INPUT_TYPE_QUERY_SCRIPT_SERVER) {
 								$action = POLLER_ACTION_SCRIPT_PHP;
 
 								$prepend = '';
@@ -410,15 +415,6 @@ function update_poller_cache($data_source, $commit = false) {
 								}
 
 								$script_path = get_script_query_path(trim($prepend . ' ' . $script_queries['arg_get'] . ' ' . $identifier . ' ' . $data_source['snmp_index']), $script_queries['script_path'] . ' ' . $script_queries['script_function'], $data_source['host_id']);
-							} elseif (($data_input['type_id'] == DATA_INPUT_TYPE_QUERY_SCRIPT_SERVER) && (!function_exists('proc_open'))) {
-								$action = POLLER_ACTION_SCRIPT;
-
-								$prepend = '';
-								if (isset($script_queries['arg_prepend']) && $script_queries['arg_prepend'] != '') {
-									$prepend = $script_queries['arg_prepend'];
-								}
-
-								$script_path = read_config_option('path_php_binary') . ' -q ' . get_script_query_path(trim($prepend . ' ' . $script_queries['arg_get'] . ' ' . $identifier . ' ' . $data_source['snmp_index']), $script_queries['script_path'], $data_source['host_id']);
 							} else {
 								$action = POLLER_ACTION_SCRIPT;
 								$script_path = get_script_query_path(trim((isset($script_queries['arg_prepend']) ? $script_queries['arg_prepend'] : '') . ' ' . $script_queries['arg_get'] . ' ' . $identifier . ' ' . $data_source['snmp_index']), $script_queries['script_path'], $data_source['host_id']);
@@ -519,14 +515,18 @@ function poller_update_poller_cache_from_buffer($local_data_ids, &$poller_items,
 		$ids = implode(', ', $local_data_ids);
 
 		if ($ids != '') {
-			db_execute("UPDATE poller_item
+			db_execute_prepared("UPDATE poller_item
 				SET present=0
-				WHERE local_data_id IN ($ids)");
+				WHERE poller_id = ?
+				AND local_data_id IN ($ids)",
+				array($poller_id));
 
 			if (($rcnn_id = poller_push_to_remote_db_connect($poller_id, true)) !== false) {
-				db_execute("UPDATE poller_item
+				db_execute_prepared("UPDATE poller_item
 					SET present=0
-					WHERE local_data_id IN ($ids)", true, $rcnn_id);
+					WHERE poller_id = ?
+					AND local_data_id IN ($ids)",
+					array($poller_id), true, $rcnn_id);
 			}
 		}
 	} else {
@@ -602,14 +602,18 @@ function poller_update_poller_cache_from_buffer($local_data_ids, &$poller_items,
 
 	/* remove stale records FROM the poller cache */
 	if ($ids != '') {
-		db_execute("DELETE FROM poller_item
-			WHERE present=0
-			AND local_data_id IN ($ids)");
+		db_execute_prepared("DELETE FROM poller_item
+			WHERE present = 0
+			AND poller_id = ?
+			AND local_data_id IN ($ids)",
+			array($poller_id));
 
 		if (($rcnn_id = poller_push_to_remote_db_connect($poller_id, true)) !== false) {
-			db_execute("DELETE FROM poller_item
-				WHERE present=0
-				AND local_data_id IN ($ids)", true, $rcnn_id);
+			db_execute_prepared("DELETE FROM poller_item
+				WHERE present = 0
+				AND poller_id = ?
+				AND local_data_id IN ($ids)",
+				array($poller_id), true, $rcnn_id);
 		}
 	} else {
 		/* only handle explicitely given local_data_ids */
@@ -623,6 +627,10 @@ function poller_update_poller_cache_from_buffer($local_data_ids, &$poller_items,
  * works on table data_input_data and poller cache
  */
 function push_out_host($host_id, $local_data_id = 0, $data_template_id = 0) {
+	global $config;
+
+	include_once($config['library_path'] . '/api_data_source.php');
+
 	/* ok here's the deal: first we need to find every data source that uses this host.
 	then we go through each of those data sources, finding each one using a data input method
 	with "special fields". if we find one, fill it will the data here FROM this host */
@@ -869,13 +877,19 @@ function utilities_get_mysql_recommendations() {
 				'value' => 'utf8mb4_unicode_ci',
 				'class' => 'warning',
 				'measure' => 'equal',
-				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8_general_ci collation type as some characters take more than a single byte.  If you are first just now installing Cacti, stop, make the changes and start over again.  If your Cacti has been running and is in production, see the internet for instructions on converting your databases and tables if you plan on supporting other languages.')
+				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8mb4_unicode_ci collation type as some characters take more than a single byte.')
+				),
+			'character_set_server' => array(
+				'value' => 'utf8mb4',
+				'class' => 'warning',
+				'measure' => 'equal',
+				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8mb4 character set as some characters take more than a single byte.')
 				),
 			'character_set_client' => array(
 				'value' => 'utf8mb4',
 				'class' => 'warning',
 				'measure' => 'equal',
-				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8 character set as some characters take more than a single byte. If you are first just now installing Cacti, stop, make the changes and start over again. If your Cacti has been running and is in production, see the internet for instructions on converting your databases and tables if you plan on supporting other languages.')
+				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8mb4 character set as some characters take more than a single byte.')
 				)
 		);
 	} else {
@@ -900,6 +914,12 @@ function utilities_get_mysql_recommendations() {
 				'measure' => 'equal',
 				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8mb4_unicode_ci collation type as some characters take more than a single byte.')
 				),
+			'character_set_server' => array(
+				'value' => 'utf8mb4',
+				'class' => 'warning',
+				'measure' => 'equal',
+				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8mb4 character set as some characters take more than a single byte.')
+				),
 			'character_set_client' => array(
 				'value' => 'utf8mb4',
 				'class' => 'warning',
@@ -1093,8 +1113,14 @@ function utilities_get_mysql_recommendations() {
 		$location = '/etc/my.cnf';
 	}
 
+	if ($database == 'MariaDB') {
+		$variables_url = 'https://mariadb.com/kb/en/server-system-variables/';
+	} else {
+		$variables_url = html_escape('https://dev.mysql.com/doc/refman/' . $link_ver . '/en/server-system-variables.html');
+	}
+
 	print '<tr class="tableHeader tableFixed">';
-	print '<th colspan="2">' . __('%s Tuning', $database) . ' (' . $location . ') - [ <a class="linkOverDark" href="' . html_escape('https://dev.mysql.com/doc/refman/' . $link_ver . '/en/server-system-variables.html') . '">' .  __('Documentation') . '</a> ] ' . __('Note: Many changes below require a database restart') . '</th>';
+	print '<th colspan="2">' . __('%s Tuning', $database) . ' (' . $location . ') - [ <a class="linkOverDark" target="_blank" href="' . $variables_url . '">' .  __('Documentation') . '</a> ] ' . __('Note: Many changes below require a database restart') . '</th>';
 	print '</tr>';
 
 	form_alternate_row();
@@ -1239,9 +1265,9 @@ function utilities_php_modules() {
 	$php_info = str_replace("\n", '', $php_info);
 	$php_info = preg_replace('/^.*\<body\>/', '', $php_info);
 	$php_info = preg_replace('/\<\/body\>.*$/', '', $php_info);
-	$php_info = preg_replace('/\<a.*\>/U', '', $php_info);
-	$php_info = preg_replace('/\<\/a\>/', '<hr>', $php_info);
+	$php_info = preg_replace('/(\<a name.*\>)([^<>]*)(\<\/a\>)/U', '$2', $php_info);
 	$php_info = preg_replace('/\<img.*\>/U', '', $php_info);
+	$php_info = preg_replace('/\<div[^<>]*\>\<\/div\>/U', '', $php_info);
 	$php_info = preg_replace('/\<\/?address\>/', '', $php_info);
 
 	return $php_info;
@@ -1511,7 +1537,7 @@ function utility_php_verify_recommends(&$recommends, $source) {
 }
 
 function utility_php_set_recommends_text(&$recs) {
-	if (is_array($recs) && sizeof($recs)) {
+	if (is_array($recs) && cacti_sizeof($recs)) {
 		foreach ($recs as $name => $recommends) {
 			if (cacti_sizeof($recommends)) {
 				foreach ($recommends as $index => $recommend) {
