@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2019 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -109,6 +109,10 @@ case 'save':
 						$errors[9] = 9;
 						$continue = false;
 					}
+				} elseif (get_nfilter_request_var($field_name) != '' && !is_valid_pathname(get_nfilter_request_var($field_name))) {
+					$_SESSION['sess_error_fields'][$field_name] = $field_name;
+					$_SESSION['sess_field_values'][$field_name] = get_nfilter_request_var($field_name);
+					$errors[36] = 36;
 				}
 
 				if ($continue) {
@@ -172,6 +176,19 @@ case 'save':
 					array($field_name));
 			}
 		} elseif (isset_request_var($field_name)) {
+			if ($field_array['method'] == 'textbox' && isset($field_array['filter'])) {
+				if (isset($field_array['options'])) {
+					$value = filter_var(get_nfilter_request_var($field_name), $field_array['filter'], $field_array['options']);
+				} else {
+					$value = filter_var(get_nfilter_request_var($field_name), $field_array['filter']);
+				}
+				if ($value === false) {
+					$_SESSION['sess_error_fields'][$field_name] = $field_name;
+					$_SESSION['sess_field_values'][$field_name] = get_nfilter_request_var($field_name);
+					$errors[3] = 3;
+					continue;
+				}
+			}
 			if (is_array(get_nfilter_request_var($field_name))) {
 				$inserts[] = '(' . db_qstr($field_name) . ', ' . db_qstr(implode(',', get_nfilter_request_var($field_name))) . ')';
 				db_execute_prepared('REPLACE INTO settings
@@ -299,6 +316,8 @@ default:
 
 	$_SESSION['sess_settings_tab'] = $current_tab;
 
+	set_request_var('tab', $current_tab);
+
 	$data_collectors = db_fetch_cell('SELECT COUNT(*) FROM poller WHERE disabled=""');
 
 	if ($data_collectors > 1) {
@@ -344,7 +363,7 @@ default:
 		$suffix = '';
 	}
 
-	html_start_box( __('Cacti Settings (%s)%s', $tabs[$current_tab], $suffix), '100%', true, '3', 'center', '');
+	html_start_box(__('Cacti Settings (%s)%s', $tabs[$current_tab], $suffix), '100%', true, '3', 'center', '');
 
 	$form_array = array();
 
@@ -402,6 +421,30 @@ default:
 		}
 	}
 
+	// Cache this setting as on large systems
+	// this query runs long
+	if ($current_tab == 'spikes') {
+		if (!isset($_SESSION['sk_templates'])) {
+			$spikekill_templates = array_rekey(
+				db_fetch_assoc('SELECT DISTINCT gt.id, gt.name
+					FROM graph_templates AS gt
+					INNER JOIN graph_templates_item AS gti
+					ON gt.id=gti.graph_template_id
+					INNER JOIN data_template_rrd AS dtr
+					ON gti.task_item_id=dtr.id
+					WHERE gti.local_graph_id=0 AND data_source_type_id IN (3,2)
+					ORDER BY name'),
+				'id', 'name'
+			);
+
+			$_SESSION['sk_templates'] = $spikekill_templates;
+		} else {
+			$spikekill_templates = $_SESSION['sk_templates'];
+		}
+
+		$form_array['spikekill_templates']['array'] = $spikekill_templates;
+	}
+
 	draw_edit_form(
 		array(
 			'config' => array('no_form_tag' => true),
@@ -433,7 +476,7 @@ default:
 			loadPageNoHeader(strURL, true, false);
 		});
 
-		$('input[value="<?php print __esc('Save');?>"]').click(function(event) {
+		$('input[value="<?php print __esc('Save');?>"]').unbind().click(function(event) {
 			event.preventDefault();
 
 			if (parseInt($('#cron_interval').val()) < parseInt($('#poller_interval').val())) {
@@ -444,12 +487,12 @@ default:
 			}
 
 			if (themeChanged != true) {
-				$.post('settings.php?tab='+$('#tab').val()+'&header=false', $('input, select, textarea').serialize()).done(function(data) {
+				$.post('settings.php?tab='+$('#tab').val()+'&header=false', $('input, select, textarea').prop('disabled', false).serialize()).done(function(data) {
 					$('#main').hide().html(data);
 					applySkin();
 				});
 			} else {
-				$.post('settings.php?tab='+$('#tab').val()+'&header=false', $('input, select, textarea').serialize()).done(function(data) {
+				$.post('settings.php?tab='+$('#tab').val()+'&header=false', $('input, select, textarea').prop('disabled', false).serialize()).done(function(data) {
 					document.location = 'settings.php?newtheme=1&tab='+$('#tab').val();
 				});
 			}
@@ -843,21 +886,21 @@ default:
 	function initSearch() {
 		if ($('#auth_method').val() == 3) {
 			switch($('#ldap_mode').val()) {
-			case "0":
+			case '0':
 				$('#row_ldap_search_base_header').hide();
 				$('#row_ldap_search_base').hide();
 				$('#row_ldap_search_filter').hide();
 				$('#row_ldap_specific_dn').hide();
 				$('#row_ldap_specific_password').hide();
 				break;
-			case "1":
+			case '1':
 				$('#row_ldap_search_base_header').show();
 				$('#row_ldap_search_base').show();
 				$('#row_ldap_search_filter').show();
 				$('#row_ldap_specific_dn').hide();
 				$('#row_ldap_specific_password').hide();
 				break;
-			case "2":
+			case '2':
 				$('#row_ldap_search_base_header').show();
 				$('#row_ldap_search_base').show();
 				$('#row_ldap_search_filter').show();
@@ -897,7 +940,8 @@ default:
 
 	function initAuth() {
 		switch($('#auth_method').val()) {
-		case "0": // None
+		case '0': // None
+			$('#row_path_basic_mapfile').hide();
 			$('#row_special_users_header').hide();
 			$('#row_auth_cache_enabled').hide();
 			$('#row_guest_user').hide();
@@ -939,7 +983,8 @@ default:
 			$('#row_secpass_lockfailed').hide();
 			$('#row_secpass_unlocktime').hide();
 			break;
-		case "1": // Builtin
+		case '1': // Builtin
+			$('#row_path_basic_mapfile').hide();
 			$('#row_special_users_header').show();
 			$('#row_auth_cache_enabled').show();
 			$('#row_guest_user').show();
@@ -981,7 +1026,8 @@ default:
 			$('#row_secpass_lockfailed').show();
 			$('#row_secpass_unlocktime').show();
 			break;
-		case "2": // Web Basic
+		case '2': // Web Basic
+			$('#row_path_basic_mapfile').show();
 			$('#row_special_users_header').show();
 			$('#row_auth_cache_enabled').hide();
 			$('#row_guest_user').show();
@@ -1023,7 +1069,8 @@ default:
 			$('#row_secpass_lockfailed').hide();
 			$('#row_secpass_unlocktime').hide();
 			break;
-		case "4": // Multiple Domains
+		case '4': // Multiple Domains
+			$('#row_path_basic_mapfile').hide();
 			$('#row_special_users_header').show();
 			$('#row_auth_cache_enabled').show();
 			$('#row_guest_user').show();
@@ -1065,7 +1112,7 @@ default:
 			$('#row_secpass_lockfailed').show();
 			$('#row_secpass_unlocktime').show();
 			break;
-		case "3": // Single Domain
+		case '3': // Single Domain
 			$('#row_special_users_header').show();
 			$('#row_auth_cache_enabled').show();
 			$('#row_guest_user').show();
@@ -1156,28 +1203,28 @@ default:
 
 	function initAvail() {
 		switch($('#availability_method').val()) {
-		case "0":
+		case '0':
 			$('#row_ping_method').hide();
 			$('#row_ping_port').hide();
 			$('#row_ping_timeout').hide();
 			$('#row_ping_retries').hide();
 			break;
-		case "1":
-		case "4":
+		case '1':
+		case '4':
 			$('#row_ping_method').show();
 			$('#row_ping_port').show();
 			$('#row_ping_timeout').show();
 			$('#row_ping_retries').show();
 			break;
-		case "3":
+		case '3':
 			$('#row_ping_method').show();
 			$('#row_ping_port').show();
 			$('#row_ping_timeout').show();
 			$('#row_ping_retries').show();
 			break;
-		case "2":
-		case "5":
-		case "6":
+		case '2':
+		case '5':
+		case '6':
 			$('#row_ping_method').hide();
 			$('#row_ping_port').hide();
 			$('#row_ping_timeout').show();
