@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2019 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -182,11 +182,17 @@ function export() {
 
 	?>
 	<script type='text/javascript'>
+	var stopTimer;
+
 	$(function() {
 		$('#export_type').change(function() {
 			strURL = 'templates_export.php?header=false&export_type='+$('#export_type').val();
 			loadPageNoHeader(strURL);
 		});
+
+		$('form#export').submit(function(event) {
+			stopTimer = setTimeout(function() { Pace.stop() }, 1000);
+		});
 	});
 	</script>
 	<?php
